# workspace 

This is a local development workspace for Trust Services API backend.
It uses docker-compose to bootstrap the services and their dependencies.

Each service has its own `.env` file in the [env](./env) directory
with environment variables which will be injected in its container. 

## Prerequisites

* Install [docker](https://docs.docker.com/engine/install/) and [docker-compose](https://docs.docker.com/compose/install/).
* [Install Go](https://golang.org/doc/install) and set the
  [`$GOPATH` variable](https://github.com/golang/go/wiki/SettingGOPATH).

## Setup

Clone the workspace repository into `$GOPATH/src/code.vereign.com/gaiax/tsa/workspace`. Using GOPATH is important
because it's the way the workspace will know where to find you Go source code.

```sh
# create the backend directory where the workspace will be checked out
mkdir -p $GOPATH/src/code.vereign.com/gaiax/tsa/workspace

# change to the created directory
cd $GOPATH/src/code.vereign.com/gaiax/tsa/workspace

# clone the workspace repo
git clone git@code.vereign.com:gaiax/tsa/workspace.git .
```

### Checkout services repos

This step is optional because you may already have the repos checked out
or you may want to do this step manually. But the workspace provides you
with a `setup.sh` script which will go and checkout the TSA backend
services at their correct locations, ready for use with `docker-compose`.

```shell
./setup.sh
```

> Note: If you have already checked out a repository, and it's not on the main branch or 
> is not in a clean state, the script will give you a warning and won't override the 
> state of your repo. You will have to checkout/update it manually.

### Usage

Now you're ready to start using the workspace environment.

```shell
# start all services
docker-compose up -d

# see the state of the containers and their exposed ports 
docker-compose ps

# follow the logs of all services in the workspace
docker-compose logs -f 

# follow the logs of specified services only
docker-compose logs -f gateway infohub

# restart a service
docker-compose restart gateway

# restart a service and reload its environment variables
docker-compose up -d gateway

# rebuild the docker image of a service
docker-compose build gateway
```

## 3rd Party Services

#### DID Resolver

The DID Resolver service can be reached at `localhost:9090` from your local machine.
Example usage:
```
curl -X GET http://localhost:9090/1.0/identifiers/did:indy:idunion:BDrEcHc8Tb4Lb2VyQZWEDE
curl -X GET http://localhost:9090/1.0/identifiers/did:key:z6Mkfriq1MqLBoPWecGoDLjguo1sB9brj6wT3qZ5BxkKpuP6
curl -X GET http://localhost:9090/1.0/identifiers/did:web:did.actor:alice
```
In order to support more DID methods refer to currently supported ones here:
```
https://github.com/decentralized-identity/universal-resolver/blob/main/docker-compose.yml
```
Needed env variables are here:
```
https://github.com/decentralized-identity/universal-resolver/blob/main/.env
```

The services access DID resolver from their containers as `uni-resolver-web:8080`


### Redis

The Redis server can be reached at `localhost:6397` from your
local machine. 

The services access Redis from their containers as `redis:6397`
